@@ -1,4 +1,4 @@
-// Copyright (C) 2006 Free Software Foundation.
+// Copyright (C) 1999-2001 Open Source Telecom Corporation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,15 +38,11 @@
 // whether to permit this exception to apply to your modifications.
 // If you do not wish that, delete this exception notice.  
 
-#include <cc++/config.h>
-#include <cc++/strchar.h>
-#include <cc++/export.h>
 #include "private.h"
-#include "audio.h"
+#include "audio2.h"
+#include <cstdio>
 
-#ifdef	CCXX_NAMESPACES
-namespace ost {
-#endif
+using namespace ost;
 
 static const char * const ErrorStrs[] = {
 	"errSuccess",
@@ -94,7 +90,7 @@ AudioCodec *AudioFile::getCodec(void)
 	}
 }
 
-void AudioFile::setShort(unsigned char *data, unsigned short val)
+void AudioFile::setShort(unsigned char *data, bin16_t val)
 {
 	if(info.order == __BIG_ENDIAN)
 	{
@@ -108,7 +104,7 @@ void AudioFile::setShort(unsigned char *data, unsigned short val)
 	}
 }
 
-unsigned short AudioFile::getShort(unsigned char *data)
+Audio::bin16_t AudioFile::getShort(unsigned char *data)
 {
 	if(info.order == __BIG_ENDIAN)
 		return data[0] * 256 + data[1];
@@ -116,24 +112,24 @@ unsigned short AudioFile::getShort(unsigned char *data)
 		return data[1] * 256 + data[0];
 }
 
-void AudioFile::setLong(unsigned char *data, unsigned long val)
+void AudioFile::setLong(unsigned char *data, bin32_t val)
 {
 	int i = 4;
 
 	while(i-- > 0)
 	{
 		if(info.order == __BIG_ENDIAN)
-			data[i] = val & 0xff;
+			data[i] = (unsigned char)(val & 0xff);
 		else
-			data[3 - i] = val & 0xff;
+			data[3 - i] = (unsigned char)(val & 0xff);
 		val /= 256;
 	}
 }
 
-unsigned long AudioFile::getLong(unsigned char * data)
+Audio::bin32_t AudioFile::getLong(unsigned char * data)
 {
 	int i = 4;
-	unsigned long val =0;
+	bin32_t val =0;
 
 	while(i-- > 0)
 	{
@@ -154,7 +150,6 @@ AudioFile::AudioFile(const char *name, unsigned long sample)
 	if(!isOpen())
 		return;
 	setPosition(sample);
-	limit = 0l;
 }
 
 AudioFile::AudioFile(const char *name, Info *info, unsigned long samples)
@@ -166,18 +161,32 @@ AudioFile::AudioFile(const char *name, Info *info, unsigned long samples)
 	if(!isOpen())
 		return;
 	setMinimum(samples);
-	limit = 0l;
 }
 
-void AudioFile::create(const char *name, Info *myinfo)
+void AudioFile::create(const char *name, Info *myinfo, bool exclusive, timeout_t framing)
 {
 	int pcm = 0;
 	unsigned char aufile[24];
 	unsigned char riffhdr[40];
+	char *ext = strrchr(name, '/');
 
-	modified = true;
+	if(!ext)
+		ext = strrchr(name, '\\');
 
-	if(!afCreate(name))
+	if(!ext)
+		ext = strrchr(name, ':');
+
+	if(!ext)
+		ext = strrchr(name, '.');
+	else
+		ext = strrchr(ext, '.');
+
+	if(!ext)
+		ext = ".none";
+
+	mode = modeWrite;
+	
+	if(!afCreate(name, exclusive))
 		return;
 
 	memset(riffhdr, 0, sizeof(riffhdr));
@@ -191,18 +200,90 @@ void AudioFile::create(const char *name, Info *myinfo)
 		strcpy(info.annotation, myinfo->annotation);
 	}
 
+	if(!stricmp(ext, ".raw") || !stricmp(ext, ".bin"))
+	{
+		info.format = raw;
+		if(info.encoding == unknownEncoding)
+			info.encoding = pcm16Mono;
+	}
+	else if(!stricmp(ext, ".au") || !stricmp(ext, ".snd"))
+	{
+		info.order = 0;
+		info.format = snd;
+	}
+	else if(!stricmp(ext, ".wav") || !stricmp(ext, ".wave"))
+	{
+		info.order = 0;
+		info.format = wave;
+	}
+	else if(!stricmp(ext, ".ul"))
+	{
+		info.encoding = mulawAudio;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 8000;
+	}
+	else if(!stricmp(ext, ".al"))
+	{
+		info.encoding = alawAudio;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 8000;
+	}
+	else if(!stricmp(ext, ".sw"))
+	{
+		info.encoding = pcm16Mono;
+		info.format = raw;
+		info.order = 0;
+	}
+	else if(!stricmp(ext, ".vox"))
+	{
+		info.encoding = voxADPCM;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 6000;
+	}
+	else if(!stricmp(ext, ".adpcm"))
+	{
+		info.encoding = g721ADPCM;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 8000;
+	}
+	else if(!stricmp(ext, ".a24"))
+	{
+		info.encoding = g723_3bit;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 8000;
+	}
+	else if(!stricmp(ext, ".a40"))
+	{
+		info.encoding = g723_5bit;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 8000;
+	}
+	else if(!stricmp(ext, ".cda"))
+	{
+		info.encoding = cdaStereo;
+		info.format = raw;
+		info.order = __LITTLE_ENDIAN;
+		info.rate = 44100;
+	}
+
 	switch(info.format)
 	{
 	case wave:
 	case riff:
-		/* 
+		/*
 		 * RIFF header
 		 *
 		 * 12 bytes
 		 *
 		 * 0-3: RIFF magic "RIFF" (0x52 49 46 46)
 		 *
-		 * 4-7: RIFF header chunk size 
+		 * 4-7: RIFF header chunk size
 		 *      (4 + (8 + subchunk 1 size) + (8 + subchunk 2 size))
 		 *
 		 * 8-11: WAVE RIFF type magic "WAVE" (0x57 41 56 45)
@@ -215,6 +296,8 @@ void AudioFile::create(const char *name, Info *myinfo)
 			strncpy((char *)riffhdr, "RIFX", 4);
 		if(!info.rate)
 			info.rate = Audio::getRate(info.encoding);
+		if(!info.rate)
+			info.rate = rate8khz;
 
 		header = 0;
 
@@ -226,7 +309,7 @@ void AudioFile::create(const char *name, Info *myinfo)
 			return;
 		}
 
-		/* 
+		/*
 		 * Subchunk 1: WAVE metadata
 		 *
 		 * Length: 24+
@@ -291,8 +374,8 @@ void AudioFile::create(const char *name, Info *myinfo)
 		else
 			setShort(riffhdr + 10, 2);
 		setLong(riffhdr + 12, info.rate);
-		setLong(riffhdr + 16, toBytes(info.encoding, info.rate));
-		setShort(riffhdr + 20, toBytes(info.encoding, 1));
+		setLong(riffhdr + 16, toBytes(info, info.rate));
+		setShort(riffhdr + 20, (bin16_t)toBytes(info, 1));
 		setShort(riffhdr + 22, 0);
 
 		switch(info.encoding)
@@ -342,6 +425,7 @@ void AudioFile::create(const char *name, Info *myinfo)
 			setShort(riffhdr + 24, 8);
 			break;
 		case gsmVoice:
+		case msgsmVoice:
 			setShort(riffhdr + 8, 0x31);
 			setShort(riffhdr + 24, 260);
 			break;
@@ -352,6 +436,7 @@ void AudioFile::create(const char *name, Info *myinfo)
 		case g723_5bit:
 			setShort(riffhdr + 8, 0x14);
 			setShort(riffhdr + 24, 5);
+		case unknownEncoding:
 		default:
 			break;
 		}
@@ -403,16 +488,18 @@ void AudioFile::create(const char *name, Info *myinfo)
 		length = getAbsolutePosition();
 		break;
 
-	case sun:
-		if(!info.order)
+	case snd:
+//		if(!info.order)
 			info.order = __BIG_ENDIAN;
 
 		if(!info.rate)
 			info.rate = Audio::getRate(info.encoding);
+		if(!info.rate)
+			info.rate = rate8khz;
 
 		strncpy((char *)aufile, ".snd", 4);
 		if(info.annotation)
-			setLong(aufile + 4, 24 + strlen(info.annotation) + 1);
+			setLong(aufile + 4, 24 + (unsigned long)strlen(info.annotation) + 1);
 		else
 			setLong(aufile + 4, 24);
 		header = getLong(aufile + 4);
@@ -464,13 +551,17 @@ void AudioFile::create(const char *name, Info *myinfo)
 			return;
 		}
 		if(info.annotation)
-			afWrite((unsigned char *)info.annotation,
-				strlen(info.annotation) + 1);
+			afWrite((unsigned char *)info.annotation, (unsigned long)strlen(info.annotation) + 1);
 		header = getAbsolutePosition();
 		length = getAbsolutePosition();
-	default:
+		break;
+	case raw:
 		break;
 	}
+	if(framing)
+		info.setFraming(framing);
+	else	
+		info.set();
 }
 
 void AudioFile::getWaveFormat(int request)
@@ -545,7 +636,7 @@ void AudioFile::getWaveFormat(int request)
 		info.encoding = g722Audio;
 		break;
 	case 0x31:
-		info.encoding = gsmVoice;
+		info.encoding = msgsmVoice;
 		break;
 	case 0x14:
 		bitsize = getLong(filehdr + 8) * 8 / info.rate;
@@ -559,69 +650,79 @@ void AudioFile::getWaveFormat(int request)
 	}
 }
 
-void AudioFile::open(const char *name)
+void AudioFile::open(const char *name, Mode m, timeout_t framing)
 {
 	unsigned char filehdr[24];
 	unsigned int count;
 	char *ext;
 	unsigned channels;
+	
+	mode = m;
 
-	modified = false;
 
-	if(!afOpen(name))
+	if(!afOpen(name, m))
 		return;
 
 	pathname = new char[strlen(name) + 1];
 	strcpy(pathname, name);
 	header = 0l;
 
+	info.framesize = 0;
+	info.framecount = 0;
 	info.encoding = mulawAudio;
 	info.format = raw;
 	info.order = 0;
 	ext = strrchr(pathname, '.');
 	if(!ext)
-		return;
+		goto done;
 
 	if(!stricmp(ext, ".ul"))
 	{
 		info.encoding = mulawAudio;
-		return;
+		goto done;
 	}
 
 	if(!stricmp(ext, ".al"))
 	{
 		info.encoding = alawAudio;
-		return;
+		goto done;
 	}
 
 	if(!stricmp(ext, ".sw") || !stricmp(ext, ".raw"))
 	{
 		info.encoding = pcm16Mono;
-		return;
+		goto done;
 	}
 
 	if(!stricmp(ext, ".vox"))
 	{
 		info.encoding = voxADPCM;
-		return;
+		goto done;
 	}
 
 	if(!stricmp(ext, ".adpcm"))
 	{
 		info.encoding = g721ADPCM;
-		return;
+		goto done;
 	}
 
 	if(!stricmp(ext, ".a24"))
 	{
 		info.encoding = g723_3bit;
-		return;
+		goto done;
 	}
 
 	if(!stricmp(ext, ".a40"))
 	{
 		info.encoding = g723_5bit;
-		return;
+		goto done;
+	}
+
+	if(!stricmp(ext, ".cda"))
+	{
+		info.order = __LITTLE_ENDIAN;
+		info.encoding = cdaStereo;
+		goto done;
 	}
 
 	strcpy((char *)filehdr, ".xxx");
@@ -671,11 +772,11 @@ void AudioFile::open(const char *name)
 
 		}
 		afSeek(header);
-		return;
+		goto done;
 	}
 	if(!strncmp((char *)filehdr, ".snd", 4))
 	{
-		info.format = sun;
+		info.format = snd;
 		info.order = __BIG_ENDIAN;
 		header = getLong(filehdr + 4);
 		info.rate = getLong(filehdr + 16);
@@ -736,42 +837,97 @@ void AudioFile::open(const char *name)
 			afSeek(24);
 			afRead((unsigned char *)info.annotation, header - 24);
 		}
-		return;
+		goto done;
 	}
 	else
 		afSeek(0);
+
+done:
+	if(framing)
+		info.setFraming(framing);
+	else
+		info.set();
+}
+
+Audio::Info::Info()
+{
+	clear();
+}
+
+void Audio::Info::clear(void)
+{
+	memset(this, 0, sizeof(Info));
+}
+
+void Audio::Info::setFraming(timeout_t timeout)
+{
+	set();
+
+	framing = getFraming(encoding);
+
+	if(!timeout)
+		return;
+
+	switch(timeout)
+	{
+	case 10:
+	case 15:
+	case 20:
+	case 30:
+	case 40:
+		break;
+	default:
+		timeout = 20;
+	}
+
+	framing = timeout;
+	framecount = (rate * framing) / 1000l;
+	framesize = toBytes(encoding, framecount);
+};
+
+void Audio::Info::set(void)
+{
+	if(!framesize)
+		framesize = getFrame(encoding);
+
+	if(!framecount)
+		framecount = getCount(encoding);
+
+	if(!rate)
+		rate = getRate(encoding);
+
+	if(!bitrate && rate && framesize && framecount)
+		bitrate = ((long)framesize * 8l * rate) / (long)framecount;
 }
 
 
 void AudioFile::close(void)
 {
-	struct stat ino;
 	unsigned char buf[58];
 	if(! isOpen())
 		return;
 
-	if(! modified)
+	if(mode != modeWrite)
 	{
 		afClose();
 		return;
 	}
 
 	if(! afSeek(0)) {
+		afClose();
 		return;
-	}
+	}	
 
-	if(afRead(buf, 58) < 58) {
+	if(-1 == afRead(buf, 58)) {
 		afClose();
 		return;
 	}
+
 	afSeek(0);
 
 	switch(info.format) {
 	case riff:
 	case wave:
-		fstat(file.fd, &ino);
-		length = ino.st_size;
-
 		// RIFF header
 		setLong(buf + 4, length - 8);
 
@@ -783,14 +939,13 @@ void AudioFile::close(void)
 			setLong(buf + 40, length - header); 
 
 		if(afWrite(buf, 58) != 58)
-		{
-			afClose();
 			return;
-		}
 
 		break;
-	default:
-		// FIXME not implemented for sun .au
+	case snd:
+		// FIXME not implemented
+		break;
+	case raw:
 		break;
 	}
 	afClose();
@@ -801,16 +956,18 @@ void AudioFile::clear(void)
 {
 	if(pathname)
 	{
-		delete[] pathname;
+		if(pathname)
+			delete[] pathname;
 		pathname = NULL;
 	}
 	if(info.annotation)
 	{
-		delete[] info.annotation;
+		if(info.annotation)
+			delete[] info.annotation;
 		info.annotation = NULL;
 	}
 	minimum = 0;
-	limit = 0;
+	iolimit = 0;
 }
 
 Audio::Error AudioFile::getInfo(Info *infobuf)
@@ -847,6 +1004,7 @@ Audio::Error AudioFile::setMinimum(unsigned long samples)
 
 unsigned AudioFile::getLinear(Linear addr, unsigned samples)
 {
+	unsigned rts = 0;
 	int count;
 	AudioCodec *codec;
 
@@ -864,15 +1022,20 @@ unsigned AudioFile::getLinear(Linear addr, unsigned samples)
 
 	count = getCount(info.encoding);
 	samples = (samples / count) * count;
-	count = toBytes(info.encoding, samples);
-	
-	char buffer[count];
+	count = toBytes(info, samples);
+
+	char *buffer = new char[count];
 	count = getBuffer(buffer, count);
 	if(count < 1)
+	{
+		delete[] buffer;
 		return 0;
+	}
 
-	samples = toSamples(info.encoding, count);
-	return codec->decode(addr, buffer, samples);
+	samples = toSamples(info, count);
+	rts = codec->decode(addr, buffer, samples);
+	delete[] buffer;
+	return rts;
 }
 
 unsigned AudioFile::putLinear(Linear addr, unsigned samples)
@@ -893,32 +1056,40 @@ unsigned AudioFile::putLinear(Linear addr, unsigned samples)
 
 	count = getCount(info.encoding);
 	samples = samples / count * count;
-	count = toBytes(info.encoding, samples);
-	
-	char buffer[count];
+	count = toBytes(info, samples);
+
+	char *buffer = new char[count];
+
 	samples = codec->encode(addr, buffer, samples);
 	if(samples < 1)
+	{
+		delete[] buffer;
 		return 0;
-	count = toBytes(info.encoding, samples);
+	}
+	count = toBytes(info, samples);
 	count = putBuffer(buffer, count);
+	delete[] buffer;
 	if(count < 0)
 		return 0;
-	return toSamples(info.encoding, count);
+	return toSamples(info, count);
 }
 
 
 int AudioFile::getBuffer(void *addr, unsigned bytes)
 {
+	Info prior;
 	char *fname;
 	unsigned char *caddr = (unsigned char *)addr;
 	int count, curpos, xfer = 0;
 
-        curpos = toBytes(info.encoding, getPosition());
-        if (limit && ((curpos + bytes) > limit))
-                bytes = limit - curpos;
+        curpos = toBytes(info, getPosition());
+        if (iolimit && ((curpos + bytes) > iolimit))
+                bytes = iolimit - curpos;
         if (bytes < 0)
                 bytes = 0;
 
+	getInfo(&prior);
+
 	for(;;)
 	{
 		count = afRead(caddr, bytes);
@@ -935,9 +1106,14 @@ int AudioFile::getBuffer(void *addr, unsigned bytes)
 		if(!fname)
 			return xfer;
 		AudioFile::close();
-		AudioFile::open(fname);
+		AudioFile::open(fname, modeRead, info.framing);
 		if(!isOpen())
 			return xfer;
+		if(prior.encoding != info.encoding)
+		{
+			AudioFile::close();
+			return xfer;
+		}
 		bytes -= count;
 		caddr += count;
 	}
@@ -973,7 +1149,7 @@ Audio::Error AudioFile::getSamples(void *addr, unsigned request)
 			break;
 
 		AudioFile::close();
-		AudioFile::open(fname);
+		AudioFile::open(fname, modeRead);
 		if(!isOpen())
 			break;
 
@@ -1004,7 +1180,7 @@ int AudioFile::putBuffer(void *addr, unsigned len)
 Audio::Error AudioFile::putSamples(void *addr, unsigned samples)
 {
 	int count;
-	int bytes = toBytes(info.encoding, samples);
+	int bytes = toBytes(info, samples);
 	if(bytes < 1)
 		return setError(errRequestInvalid);
 
@@ -1042,12 +1218,12 @@ Audio::Error AudioFile::setLimit(unsigned long samples)
 
 	if(!samples)
 	{
-		limit = 0;
+		iolimit = 0;
 		return errSuccess;
 	}
 
 	samples += getPosition();
-	limit = toBytes(info.encoding, samples);
+	iolimit = toBytes(info, samples);
 	return errSuccess;
 }
 
@@ -1055,26 +1231,25 @@ bool AudioFile::isSigned(void)
 {
 	switch(info.format)
 	{
-	case sun:
+	case snd:
 		return false;
 	default:
-		switch(info.encoding)
-		{
-		case pcm8Mono:
-		case pcm8Stereo:
-		case pcm16Mono:
-		case pcm16Stereo:
-		case pcm32Mono:
-		case pcm32Stereo:
-			return true;
-		default:
-			return false;
-		}
+		break;
+	}
+
+	switch(info.encoding)
+	{
+	case pcm8Mono:
+	case pcm8Stereo:
+	case pcm16Mono:
+	case pcm16Stereo:
+	case pcm32Mono:
+	case pcm32Stereo:
+		return true;
+	default:
+		return false;
 	}
 }
 
-#ifdef	CCXX_NAMESPACES
-};
-#endif
 
 
